@@ -1,11 +1,10 @@
 /// name = "light_phylogeny"
-/// version = "0.20.0"
+/// version = "1.4.1"
 /// authors = ["Simon Penel <simon.penel@univ-lyon1.fr>"]
 /// license = "CECILL-2.1"
 
 // Convention: "pipe" trees are equivalent to "upper" trees, "path" trees are equivalenet to "lower" trees
 
-
 use log::{info};
 use std::fs;
 use crate::arena::Options;
@@ -42,7 +41,7 @@ pub fn read_newick(filename:String, tree: &mut ArenaTree<String>) {
     newick2tree(contents, tree, root, &mut 0);
 }
 /// Read a phyloxml file and store the tree into a ArenaTree structure.
-pub fn read_phyloxml(filename:String, tree: &mut ArenaTree<String>) {
+pub fn read_phyloxml(filename: String, tree: &mut ArenaTree<String>) {
     let contents = fs::read_to_string(filename);
     let contents = match contents {
         Ok(contents) => contents,
@@ -74,23 +73,27 @@ pub fn read_phyloxml(filename:String, tree: &mut ArenaTree<String>) {
             // Create the node, get its associated index and store it in root again
             let root = tree.new_node(root.to_string());
             // Call xlm2tree on the root
-            xml2tree(node, root, &mut index,  tree);
+            xml2tree(node, root, &mut index, tree);
             break;
         }
     }
     info!("Tree : {:?}",tree);
 }
 /// Adding invisible parent nodes in order to display multiple species trees.
-pub fn add_virtual_roots(global_roots: &mut std::vec::Vec<String>, pere: &mut usize, level: &mut usize,
-    global_pipe: &mut ArenaTree<String>) {
+pub fn add_virtual_roots(
+    global_roots: &mut std::vec::Vec<String>,
+    pere: &mut usize,
+    level: &mut usize,
+    global_pipe: &mut ArenaTree<String>,
+    ) {
     if global_roots.len() == 1 {
-        let nom_droite = "VIRTUAL_".to_string().to_owned()+&level.to_string()+"_"+&(*pere-1).to_string();
-        let nom_pere = "ROOT_".to_string().to_owned()+&level.to_string()+"_"+&(*pere).to_string();
+        let nom_droite = "VIRTUAL_".to_string().to_owned() + &level.to_string() + "_"+&(*pere-1).to_string();
+        let nom_pere = "ROOT_".to_string().to_owned() + &level.to_string() + "_"+&(*pere).to_string();
         let  node_pere = global_pipe.new_node(nom_pere.clone());
-        global_pipe.arena[node_pere].name=nom_pere;
+        global_pipe.arena[node_pere].name = nom_pere;
         global_pipe.arena[node_pere].visible = false;
         let  node_droite = global_pipe.node(nom_droite.clone());
-        global_pipe.arena[node_droite].name=nom_droite;
+        global_pipe.arena[node_droite].name = nom_droite;
         global_pipe.arena[node_droite].visible = false;
         let node_left = global_pipe.node(global_roots[0].to_string());
         global_pipe.arena[node_pere].children.push(node_left);
@@ -98,16 +101,16 @@ pub fn add_virtual_roots(global_roots: &mut std::vec::Vec<String>, pere: &mut us
         global_pipe.arena[node_left].parent = Some(node_pere);
         global_pipe.arena[node_droite].parent = Some(node_pere);
         if *pere >  0 {
-            let mut my_vect : Vec<String> = (0..(*pere+1)).map(|n| "ROOT_".to_string().to_owned()+&level.to_string()+"_"+&n.to_string()).collect();
+            let mut my_vect : Vec<String> = (0..(*pere+1)).map(|n| "ROOT_".to_string().to_owned() + &level.to_string() + "_" + &n.to_string()).collect();
             *level = *level + 1;
             *pere = 0;
-            add_virtual_roots( &mut my_vect, pere, level, global_pipe);
+            add_virtual_roots(&mut my_vect, pere, level, global_pipe);
         }
     }
     if global_roots.len() == 2 {
-        let nom_pere = "ROOT_".to_string().to_owned()+&level.to_string()+"_"+&(*pere).to_string();
+        let nom_pere = "ROOT_".to_string().to_owned() + &level.to_string() + "_"+&(*pere).to_string();
         let  node_pere = global_pipe.node(nom_pere.clone());
-        global_pipe.arena[node_pere].name=nom_pere;
+        global_pipe.arena[node_pere].name = nom_pere;
         global_pipe.arena[node_pere].visible = false;
         let  node_droite = global_pipe.node(global_roots[1].to_string());
         let node_left = global_pipe.node(global_roots[0].to_string());
@@ -115,20 +118,20 @@ pub fn add_virtual_roots(global_roots: &mut std::vec::Vec<String>, pere: &mut us
         global_pipe.arena[node_pere].children.push(node_droite);
         global_pipe.arena[node_left].parent = Some(node_pere);
         global_pipe.arena[node_droite].parent = Some(node_pere);
-        if *pere >  0 {
-            let mut my_vect : Vec<String> = (0..(*pere+1)).map(|n| "ROOT_".to_string().to_owned()+&level.to_string()+"_"+&n.to_string()).collect();
+        if *pere > 0 {
+            let mut my_vect : Vec<String> = (0..(*pere+1)).map(|n| "ROOT_".to_string().to_owned() + &level.to_string() + "_" + &n.to_string()).collect();
             *level = *level + 1;
             *pere = 0;
-            add_virtual_roots( &mut my_vect, pere, level, global_pipe);
+            add_virtual_roots(&mut my_vect, pere, level, global_pipe);
         }
     }
     if global_roots.len() > 2 {
         let  (tete, queue) = global_roots.split_at(2);
-        let nom_pere = "ROOT_".to_string().to_owned()+&level.to_string()+"_"+&(*pere).to_string();
+        let nom_pere = "ROOT_".to_string().to_owned() + &level.to_string() + "_"+&(*pere).to_string();
         let  node_pere = global_pipe.node(nom_pere.clone());
-        global_pipe.arena[node_pere].name=nom_pere;
+        global_pipe.arena[node_pere].name = nom_pere;
         global_pipe.arena[node_pere].visible = false;
-        let  node_droite = global_pipe.node(tete[1].to_string());
+        let node_droite = global_pipe.node(tete[1].to_string());
         let node_left = global_pipe.node(tete[0].to_string());
         global_pipe.arena[node_pere].children.push(node_left);
         global_pipe.arena[node_pere].children.push(node_droite);
@@ -136,13 +139,16 @@ pub fn add_virtual_roots(global_roots: &mut std::vec::Vec<String>, pere: &mut us
         global_pipe.arena[node_droite].parent = Some(node_pere);
         let mut queue_clone = queue.clone().to_vec();
         *pere = *pere + 1;
-        add_virtual_roots( &mut queue_clone, pere,level, global_pipe);
+        add_virtual_roots(&mut queue_clone, pere,level, global_pipe);
     }
 }
 /// Read a recphyloxml file and store the species and gene trees into several ArenaTree structures.
-pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<String>,
+pub fn read_recphyloxml_multi(
+    filename: String,
+    global_pipe: &mut ArenaTree<String>,
     gene_trees: &mut std::vec::Vec<ArenaTree<String>>,
-    global_roots:  &mut std::vec::Vec<usize>) {
+    global_roots: &mut std::vec::Vec<usize>
+    ) {
     let contents = fs::read_to_string(filename);
     let contents = match contents {
         Ok(contents) => contents,
@@ -176,7 +182,7 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
              std::process::exit(1);
         },
     };
-    let  mut index  =  &mut 0;
+    let mut index = &mut 0;
     let mut global_root_names: std::vec::Vec<String> = Vec::new();
     for spnode in spnodes {
         info!("Search spTree node {:?}",spnode);
@@ -186,13 +192,13 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
         let descendants = spnode.descendants();
         // Search for the first occurence of the "clade" tag
         // index = &(index + 1) ;
-        *index +=1;
+        *index += 1;
         for node in descendants {
             if node.has_tag_name("clade"){
-                let globalroot = "G".to_owned()+&index.to_string();
+                let globalroot = "G".to_owned() + &index.to_string();
                 global_root_names.push(globalroot.clone());
                 // Create the node, get its associated index and store it in root again
-                info!("Create {}","G".to_owned()+&index.to_string());
+                info!("Create {}","G".to_owned() + &index.to_string());
                 let globalroot = global_pipe.new_node(globalroot.to_string());
                 global_roots.push(globalroot);
                 // Call xlm2tree on the root
@@ -202,7 +208,7 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
         }
     }
     if global_roots.len() > 1 {
-        add_virtual_roots(&mut global_root_names, &mut 0, &mut 0,  global_pipe);
+        add_virtual_roots(&mut global_root_names, &mut 0, &mut 0, global_pipe);
     }
     let  nb_sptree =  global_roots.len().clone();
     println!("Number of species trees : {}",nb_sptree);
@@ -232,7 +238,7 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
             if node.has_tag_name("clade"){
                 // The first clade is the root
                 // Initialize the index used for defining the value
-                let mut index  = &mut 0;
+                let mut index = &mut 0;
                 // Val of the root
                 let root = "N".to_owned()+&index.to_string();
                 // Create the node, get its associated index and store it in root again
@@ -247,7 +253,7 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
         // Ajoute l'arbre de gene
         gene_trees.push(gene_tree);
     }
-    let  nb_gntree =  gene_trees.len().clone();
+    let  nb_gntree = gene_trees.len().clone();
     println!("Number of gene trees : {}",nb_gntree);
     info!("List of gene trees : {:?}",gene_trees);
 }
@@ -270,7 +276,7 @@ pub fn phyloxml_processing(
     // profondeur => Y, left => X= 0, right X=1
     // ---------------------------------------------------------
     let  root = tree.get_root();
-    knuth_layout(&mut tree,root, &mut 1);
+    knuth_layout(&mut tree, root, &mut 1);
     // ---------------------------------------------------------
     // Option : Cladogramme
     // ---------------------------------------------------------
@@ -294,11 +300,11 @@ pub fn phyloxml_processing(
     // ---------------------------------------------------------
     // OPTIONAL Scale the heigt if needed
     // ---------------------------------------------------------
-    if options.height != 1.0 { scale_heigth(&mut tree,options.height)};
+    if options.height != 1.0 {scale_heigth(&mut tree,options.height)};
     // ---------------------------------------------------------
     // OPTIONAL Scale the width if needed
     // ---------------------------------------------------------
-    if options.width != 1.0 { scale_width(&mut tree,options.width)};
+    if options.width != 1.0 {scale_width(&mut tree,options.width)};
     // ---------------------------------------------------------
     // Option : real_length
     // ---------------------------------------------------------
@@ -322,9 +328,9 @@ pub fn recphyloxml_processing(
     mut gene_trees: &mut std::vec::Vec<ArenaTree<String>>,  // gene trees
     mut options: &mut Options,                              // display options
     config: &Config,                                        // svg configuration
-    mapping:bool,                                           // map gene and species
+    mapping: bool,                                           // map gene and species
     transfers: & std::vec::Vec<(String,String)>,            // optional additional transfers
-    outfile: String                                         // output file
+    outfile: String,                                         // output file
 ) {
     // -----------------------
     // Traitement en 12 etapes
@@ -348,16 +354,16 @@ pub fn recphyloxml_processing(
             Err(_err) => {
                 info!("[recphyloxml_processing] No FREE_LIVING_ROOT  in the tree, I create one.");
                 let left = sp_tree.get_root();
-                let  right = sp_tree.new_node("free_living".to_string());
-                sp_tree.arena[right].name="FREE_LIVING".to_string();
+                let right = sp_tree.new_node("free_living".to_string());
+                sp_tree.arena[right].name = "FREE_LIVING".to_string();
                 free_root = right;
-                let  fl_root = sp_tree.new_node("free_living_root".to_string());
-                sp_tree.arena[fl_root].name="FREE_LIVING_ROOT".to_string();
-                sp_tree.arena[fl_root].visible=false;
+                let fl_root = sp_tree.new_node("free_living_root".to_string());
+                sp_tree.arena[fl_root].name = "FREE_LIVING_ROOT".to_string();
+                sp_tree.arena[fl_root].visible = false;
                 sp_tree.arena[fl_root].children.push(left);
                 sp_tree.arena[fl_root].children.push(right);
-                sp_tree.arena[right].parent=Some(fl_root);
-                sp_tree.arena[left].parent=Some(fl_root);
+                sp_tree.arena[right].parent = Some(fl_root);
+                sp_tree.arena[left].parent = Some(fl_root);
                 info!("[recphyloxml_processing] FREE_LIVING_ROOT : {}",free_root);
             },
         };
@@ -366,7 +372,7 @@ pub fn recphyloxml_processing(
     // 1ere étape :initialisation des x,y de l'arbre d'espèces :
     // profondeur => Y, left => X= 0, right X=1
     // ---------------------------------------------------------
-    let  root = sp_tree.get_root();
+    let root = sp_tree.get_root();
     knuth_layout(&mut sp_tree,root, &mut 1);
     // --------------------
     // OPTIONAL  Cladogramme
@@ -380,7 +386,7 @@ pub fn recphyloxml_processing(
     // chaque noeud de l'arbre d'espèces
     // ---------------------------------------------------------
     if mapping {
-        map_species_trees(&mut sp_tree,&mut gene_trees);
+        map_species_trees(&mut sp_tree, &mut gene_trees);
         info!("Species tree after mapping : {:?}",sp_tree);
     }
     //  Option: uniformise les noeuds de l'arbre d'espece
@@ -389,7 +395,7 @@ pub fn recphyloxml_processing(
         // Il fait decaler le tout (mais je ne comprend pas pourquoi?)
         let smallest_y = sp_tree.get_smallest_y();
         let root_width = sp_tree.arena[root].nbg as f32 * PIPEBLOCK ;
-        shift_nodes_y_values(sp_tree,root,-smallest_y + root_width);
+        shift_nodes_y_values(sp_tree, root, -smallest_y + root_width);
     }
     // ---------------------------------------------------------
     // Option : utilise les longueurs de branches
@@ -407,13 +413,12 @@ pub fn recphyloxml_processing(
         // On utilise le nombre de gene , qui a etet uniformise precedemment
         // options.scale = (sp_tree.arena[root].nbg as f32 / 2.0  + 0.25)  / min_dist ;
         let optimised_factor = (sp_tree.arena[root].nbg as f32 / 2.0  + 0.25)  / min_dist ;
-        options.scale = optimised_factor  * options.scale ;
-
+        options.scale = optimised_factor * options.scale ;
         real_length(&mut sp_tree, root, &mut 0.0, & options);
         // Il faut decaler le tout (mais je ne comprend pas pourquoi?)
         let smallest_y = sp_tree.get_smallest_y();
         let root_width = sp_tree.arena[root].nbg as f32 * PIPEBLOCK ;
-        shift_nodes_y_values(sp_tree,root,-smallest_y + root_width);
+        shift_nodes_y_values(sp_tree, root, -smallest_y + root_width);
     }
     // ---------------------------------------------------------
     // OPTIONAL Optimisation if needed
@@ -427,7 +432,7 @@ pub fn recphyloxml_processing(
         // Analyse des transfers
         // ---------------------
         let gene_transfers = get_gtransfer(&mut gene_trees[0]);
-        let selected_transfers =gene_transfers;
+        let selected_transfers = gene_transfers;
         println!("Optimisation: Gene transfers = {:?}",selected_transfers);
         let nbt = selected_transfers.len();
         println!("Optimisation: Number of transfers = {:?}",nbt);
@@ -452,7 +457,7 @@ pub fn recphyloxml_processing(
     check_contour_postorder(&mut sp_tree, root);
     // ---------------------------------------------------------
     // 4eme étape : Décale toutes les valeurs de x en fonction
-    // de xmod dans l'abre d'espèces
+    // de xmod dans l'arbre d'espèces
     // ---------------------------------------------------------
     shift_mod_xy(&mut sp_tree, root, &mut 0.0, &mut 0.0);
     // ---------------------------------------------------------
@@ -477,42 +482,42 @@ pub fn recphyloxml_processing(
     if ! options.real_length_flag {
         // Egalise les feuilles
         let largest_y  = sp_tree.get_largest_y();
-        set_leaves_y_values(sp_tree,root,largest_y);
+        set_leaves_y_values(sp_tree, root, largest_y);
     }
     // ---------------------------------------------------------
     // OPTIONAL Scale the heigt if needed
     // ---------------------------------------------------------
-    if options.height != 1.0 { scale_heigth(&mut sp_tree,options.height)};
+    if options.height != 1.0 {scale_heigth(&mut sp_tree, options.height)};
     // ---------------------------------------------------------
     // OPTIONAL Scale the width if needed
     // ---------------------------------------------------------
-    if options.width != 1.0 { scale_width(&mut sp_tree,options.width)};
+    if options.width != 1.0 { scale_width(&mut sp_tree, options.width)};
     // ---------------------------------------------------------
     // 8ème etape : décale les noeuds de gene associés à un
     // noeud d'especes pour éviter qu'ils soit superposés
     // ---------------------------------------------------------
-    bilan_mappings(&mut sp_tree, &mut gene_trees,initial_root, & options);
+    bilan_mappings(&mut sp_tree, &mut gene_trees, initial_root, & options);
     // ---------------------------------------------------------
     // 9eme etape : centre les noeuds de genes dans le noeud de l'espece
     // ---------------------------------------------------------
-    center_gene_nodes(&mut sp_tree,&mut gene_trees,initial_root);
+    center_gene_nodes(&mut sp_tree,&mut gene_trees, initial_root);
     // ---------------------------------------------------------
     // 10eme etape traite spécifiquement les duplications et les feuilles
     // ---------------------------------------------------------
-    move_dupli_mappings(&mut sp_tree, &mut gene_trees,initial_root);
+    move_dupli_mappings(&mut sp_tree, &mut gene_trees, initial_root);
     // 11eme etape : Ca des speciations dont les fils sont
     // dans le meme noeud "tuyeau" que le pere
     // Cela n'arrice que quand on mappe des genes sur des hotes
     // via les parasites (thirdlevel)
     // ---------------------------------------------------------
-    move_species_mappings(&mut sp_tree, &mut gene_trees,initial_root);
+    move_species_mappings(&mut sp_tree, &mut gene_trees, initial_root);
     // Option : traitement specifique des gene "free living"
     if options.free_living {
-        process_fl(&mut sp_tree, &mut gene_trees,free_root, &options);
+        process_fl(&mut sp_tree, &mut gene_trees, free_root, &options);
     }
     if ! options.real_length_flag {
         // Egalise les feuilles 2
-        uniformise_gene_leaves_y_values(sp_tree,gene_trees);
+        uniformise_gene_leaves_y_values(sp_tree, gene_trees);
     }
     // ---------------------------------------------------------
     // Fin: Ecriture du fichier svg
@@ -524,7 +529,7 @@ pub fn recphyloxml_processing(
                 options.gene_internal = true;
                 }
             draw_tree(&mut sp_tree, outfile, &options, &config);
-            },
+        },
         false => draw_sptree_gntrees(&mut sp_tree, &mut gene_trees, outfile,
             &options, &config, &transfers),
     };
